from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = ["{{first_name}} {{last_name}}"]

    first_names_male = [
        "Aldo",
        "Alen",
        "Andrija",
        "Ante",
        "Anto",
        "Anton",
        "Antonio",
        "Antun",
        "Boris",
        "Božo",
        "Branko",
        "Bruno",
        "Dalibor",
        "Damir",
        "Danijel",
        "Dario",
        "Darko",
        "David",
        "Davor",
        "Dejan",
        "Denis",
        "Dino",
        "Domagoj",
        "Dominik",
        "Dragan",
        "Dragutin",
        "Dražen",
        "Duje",
        "Dušan",
        "Elvis",
        "Erik",
        "Filip",
        "Fran",
        "Franjo",
        "Goran",
        "Hrvoje",
        "Igor",
        "Ilija",
        "Ivan",
        "Ivica",
        "Ivo",
        "Jakov",
        "Janko",
        "Josip",
        "Joso",
        "Jozo",
        "Joško",
        "Juraj",
        "Jure",
        "Karlo",
        "Kristijan",
        "Leon",
        "Lovre",
        "Lovro",
        "Luka",
        "Marijan",
        "Marin",
        "Mario",
        "Marko",
        "Martin",
        "Mate",
        "Matej",
        "Mateo",
        "Matija",
        "Mato",
        "Mihael",
        "Mijo",
        "Milan",
        "Mile",
        "Mirko",
        "Miroslav",
        "Mislav",
        "Mladen",
        "Nenad",
        "Niko",
        "Nikola",
        "Nikša",
        "Patrik",
        "Pavao",
        "Pero",
        "Petar",
        "Robert",
        "Roko",
        "Romano",
        "Rudolf",
        "Saša",
        "Siniša",
        "Slavko",
        "Stjepan",
        "Tomislav",
        "Tomo",
        "Toni",
        "Valter",
        "Vedran",
        "Viktor",
        "Vjekoslav",
        "Vladimir",
        "Vlado",
        "Zdravko",
        "Zlatko",
        "Zoran",
        "Zvonko",
        "Đuro",
        "Šime",
        "Željko",
    ]

    first_names_female = [
        "Albina",
        "Ana",
        "Andrea",
        "Ane",
        "Anica",
        "Anita",
        "Anka",
        "Ankica",
        "Antonija",
        "Anđa",
        "Anđela",
        "Bara",
        "Barbara",
        "Barica",
        "Biljana",
        "Biserka",
        "Božena",
        "Božica",
        "Branka",
        "Danica",
        "Danijela",
        "Dora",
        "Draga",
        "Dragica",
        "Elizabeta",
        "Ema",
        "Eva",
        "Franciska",
        "Fuma",
        "Gordana",
        "Hana",
        "Ika",
        "Iva",
        "Ivana",
        "Ivanka",
        "Ivka",
        "Jadranka",
        "Jana",
        "Janja",
        "Jasminka",
        "Jasna",
        "Jele",
        "Jelena",
        "Josipa",
        "Julijana",
        "Karla",
        "Kata",
        "Katarina",
        "Kate",
        "Katica",
        "Kristina",
        "Lana",
        "Lara",
        "Laura",
        "Lea",
        "Lidija",
        "Ljiljana",
        "Ljubica",
        "Lorena",
        "Lucija",
        "Maja",
        "Manda",
        "Mara",
        "Mare",
        "Maria",
        "Marica",
        "Marija",
        "Marijana",
        "Marina",
        "Marta",
        "Martina",
        "Matea",
        "Mateja",
        "Matija",
        "Mia",
        "Milena",
        "Milica",
        "Milka",
        "Mira",
        "Mirjana",
        "Nada",
        "Nataša",
        "Nevenka",
        "Nika",
        "Nikolina",
        "Nina",
        "Petra",
        "Renata",
        "Rozalija",
        "Ruža",
        "Ružica",
        "Sandra",
        "Sanja",
        "Sara",
        "Slavica",
        "Snježana",
        "Stana",
        "Suzana",
        "Tatjana",
        "Tea",
        "Terezija",
        "Valentina",
        "Vera",
        "Veronika",
        "Vesna",
        "Zdenka",
        "Zora",
        "Zorka",
        "Đurđica",
        "Štefanija",
        "Štefica",
        "Željka",
    ]

    first_names = first_names_female + first_names_male
    last_names = [
        "Abičić",
        "Abramović",
        "Adžijević",
        "Akmačić",
        "Alković",
        "Amanović",
        "Amidžić",
        "Andrašek",
        "Andrijašević",
        "Andrijević",
        "Aničić",
        "Antić",
        "Antolković",
        "Antonić",
        "Babić",
        "Bajan",
        "Baksa",
        "Balatinac",
        "Balinčić",
        "Balić",
        "Baljak",
        "Ban",
        "Baničević",
        "Banko",
        "Banovac",
        "Barac",
        "Barać",
        "Barbieri",
        "Barbir",
        "Barbić",
        "Barešić",
        "Barić",
        "Baričević",
        "Barišić",
        "Barković",
        "Barnaba",
        "Barušić",
        "Batrac",
        "Batrnek",
        "Bazjak",
        "Bačak",
        "Bačić",
        "Bašić",
        "Bašnec",
        "Bebić",
        "Begonja",
        "Beletić",
        "Belošević",
        "Benić",
        "Benčić",
        "Beraković",
        "Bernardić",
        "Bertoša",
        "Bezić",
        "Bijelić",
        "Bingula",
        "Birkić",
        "Birtić",
        "Bićanić",
        "Bičak",
        "Bičanić",
        "Bjeliš",
        "Blagaić",
        "Blažek",
        "Blažević",
        "Blažičko",
        "Boban",
        "Bobanović",
        "Bogadi",
        "Bogdan",
        "Bogović",
        "Bojanić",
        "Boljkovac",
        "Bolčević",
        "Borak",
        "Borojević",
        "Borošak",
        "Bosilj",
        "Botica",
        "Botić",
        "Bošnjak",
        "Bošnjaković",
        "Božanić",
        "Božanović",
        "Božiković",
        "Božić",
        "Božičević",
        "Božičković",
        "Bradić",
        "Brajković",
        "Bralić",
        "Brandić",
        "Branković",
        "Bračun",
        "Brcko",
        "Brezjan",
        "Britvec",
        "Brlas",
        "Brlek",
        "Brletić",
        "Bukvić",
        "Buljan",
        "Bungić",
        "Bunić",
        "Bunčić",
        "Burić",
        "Burčul",
        "Buršić",
        "Butković",
        "Buzov",
        "Bučanac",
        "Bučar",
        "Bušljeta",
        "Cafuk",
        "Car",
        "Carić",
        "Cestarić",
        "Ciganović",
        "Cik",
        "Cindrić",
        "Colić",
        "Crevar",
        "Crneković",
        "Crnković",
        "Cukon",
        "Culi",
        "Cvenić",
        "Cvetković",
        "Cvetnić",
        "Cvitan",
        "Cvrtila",
        "Dabo",
        "Damijanić",
        "Damjanović",
        "Darojković",
        "Dautanec",
        "Dautović",
        "Debelić",
        "Derežić",
        "Deže",
        "Didović",
        "Dizdar",
        "Dobrić",
        "Dolinar",
        "Dolić",
        "Dominiković",
        "Dominković",
        "Dončević",
        "Dragičević",
        "Dragobratović",
        "Dragojević",
        "Drakulić",
        "Drandić",
        "Dražić",
        "Dretvić",
        "Dubovečak",
        "Dujmović",
        "Dukić",
        "Duvančić",
        "Dvojak",
        "Džapo",
        "Erceg",
        "Ercegović",
        "Ergotić",
        "Eršek",
        "Eterović",
        "Fabijanić",
        "Ferenac",
        "Filar",
        "Filipović",
        "Filipčić",
        "Flego",
        "Forjan",
        "Franić",
        "Franičević",
        "Franjić",
        "Franušić",
        "Friščić",
        "Frketić",
        "Fuček",
        "Gabud",
        "Galešić",
        "Galić",
        "Galović",
        "Gegić",
        "Gelemanović",
        "Glasnović",
        "Glavan",
        "Glavor",
        "Gligora",
        "Godinić",
        "Golubić",
        "Gotić",
        "Govorčinović",
        "Gracin",
        "Grba",
        "Grbac",
        "Gredičak",
        "Gregov",
        "Grgić",
        "Grgurević",
        "Grgurić",
        "Grgurovac",
        "Grubišić",
        "Gržetić",
        "Gržinčić",
        "Guberović",
        "Gudelj",
        "Gulan",
        "Guštin",
        "Hadrović",
        "Hadžić",
        "Halambek",
        "Halapir",
        "Hanžek",
        "Harapin",
        "Hardi",
        "Herceg",
        "Herout",
        "Hećimović",
        "Hinić",
        "Hodak",
        "Horak",
        "Horvat",
        "Horvatek",
        "Horvatinec",
        "Horvatić",
        "Hrabar",
        "Hranić",
        "Hranj",
        "Hrastinski",
        "Hren",
        "Hrvojić",
        "Hršak",
        "Hrženjak",
        "Huljev",
        "Husnjak",
        "Ignac",
        "Ilijić",
        "Ilinović",
        "Ilić",
        "Ivandić",
        "Ivanović",
        "Ivančan",
        "Ivšić",
        "Jagarinec",
        "Jagečić",
        "Jakopec",
        "Jakopović",
        "Jakovac",
        "Jaković",
        "Jakovljević",
        "Jakupić",
        "Jakuš",
        "Jasprica",
        "Jelavić",
        "Jelić",
        "Jemrić",
        "Jerković",
        "Jonjić",
        "Josipović",
        "Jovanovac",
        "Jovanović",
        "Jović",
        "Jozić",
        "Jugovac",
        "Jukić",
        "Juretić",
        "Jureško",
        "Jurić",
        "Jurišić",
        "Jurjević",
        "Jurković",
        "Jurlina",
        "Jurčić",
        "Kadija",
        "Kahlina",
        "Kalanjoš",
        "Kalazić",
        "Kaniški",
        "Karagić",
        "Karamarko",
        "Karlović",
        "Karmelić",
        "Karuza",
        "Katić",
        "Kauzlarić",
        "Keleković",
        "Kelečić",
        "Kelić",
        "Kereković",
        "Kevo",
        "Kinkela",
        "Kirinčić",
        "Klanac",
        "Klarin",
        "Klarić",
        "Klasić",
        "Kligl",
        "Knezović",
        "Knežević",
        "Kocijančić",
        "Kokanović",
        "Kokorić",
        "Kolarec",
        "Kolega",
        "Kolić",
        "Komar",
        "Komljenović",
        "Kopjar",
        "Kos",
        "Kosanović",
        "Kosić",
        "Kostanić",
        "Kostelac",
        "Kovač",
        "Kovaček",
        "Kovačević",
        "Kovačić",
        "Koški",
        "Koščević",
        "Krajcar",
        "Kralj",
        "Kraljević",
        "Kraljić",
        "Kramarić",
        "Kresonja",
        "Križan",
        "Krpan",
        "Krznarić",
        "Krčelić",
        "Kršanac",
        "Krželj",
        "Kujundžić",
        "Kukučka",
        "Kunac",
        "Kupsjak",
        "Kurtoić",
        "Kuveždić",
        "Kuzmić",
        "Kučić",
        "Kuščević",
        "Labaš",
        "Labinjan",
        "Ladavac",
        "Lakošeljac",
        "Lasić",
        "Lazar",
        "Legović",
        "Lelas",
        "Lenić",
        "Lešić",
        "Lešković",
        "Leščić",
        "Liber",
        "Licul",
        "Liović",
        "Lisica",
        "Ljubetić",
        "Ljubić",
        "Ljubičić",
        "Lojen",
        "Lorencin",
        "Lovrić",
        "Lucić",
        "Lukinić",
        "Lukić",
        "Lukša",
        "Lukšić",
        "Lučić",
        "Macan",
        "Madunić",
        "Magić",
        "Mahnet",
        "Majdenić",
        "Majstorović",
        "Makovac",
        "Maletić",
        "Malnar",
        "Maloča",
        "Mamić",
        "Mamula",
        "Maras",
        "Marasović",
        "Maraš",
        "Mardešić",
        "Maretić",
        "Marijanović",
        "Marin",
        "Marinković",
        "Marinović",
        "Marić",
        "Maričević",
        "Marjanović",
        "Markanjević",
        "Markovac",
        "Marković",
        "Markuš",
        "Martinić",
        "Martinović",
        "Martinčević",
        "Martić",
        "Marušić",
        "Maršić",
        "Maržić",
        "Matahlija",
        "Matana",
        "Matas",
        "Mateša",
        "Matijaš",
        "Matijević",
        "Matić",
        "Matko",
        "Matković",
        "Matokanović",
        "Matovina",
        "Matošević",
        "Matulin",
        "Matulić",
        "Mavra",
        "Maćešić",
        "Medač",
        "Medić",
        "Medved",
        "Meić",
        "Merkaš",
        "Mesarić",
        "Mesić",
        "Mihalić",
        "Mihaljević",
        "Mihelčić",
        "Mihić",
        "Mikić",
        "Miklečić",
        "Mikulandra",
        "Mikulec",
        "Mikulčić",
        "Milas",
        "Milatić",
        "Miletić",
        "Milevoj",
        "Milina",
        "Miličić",
        "Miloslavić",
        "Milotić",
        "Milovac",
        "Mimica",
        "Miočić",
        "Mirosavljević",
        "Mirt",
        "Mičetić",
        "Mišak",
        "Miše",
        "Mišković",
        "Modrić",
        "Mofardin",
        "Morić",
        "Moscarda",
        "Moslavac",
        "Močibob",
        "Mraović",
        "Mudri",
        "Mudronja",
        "Mustapić",
        "Mustač",
        "Mušćet",
        "Mužina",
        "Mužić",
        "Nakić",
        "Načinović",
        "Nedić",
        "Nikolić",
        "Nimac",
        "Nišević",
        "Nižetić",
        "Norac",
        "Novak",
        "Novosel",
        "Obradović",
        "Obratov",
        "Odobašić",
        "Orbanić",
        "Orešković",
        "Orlić",
        "Orlović",
        "Oršolić",
        "Oršoš",
        "Oršulić",
        "Ozimec",
        "Pajur",
        "Palić",
        "Pamić",
        "Pandurić",
        "Papak",
        "Paparić",
        "Paradi",
        "Pavelić",
        "Pavin",
        "Pavić",
        "Pavičić",
        "Pavković",
        "Pavlović",
        "Pecotić",
        "Pedišić",
        "Peharda",
        "Penić",
        "Perak",
        "Perić",
        "Perišić",
        "Perkov",
        "Perković",
        "Peroš",
        "Peruško",
        "Perčić",
        "Petek",
        "Peterlik",
        "Petrić",
        "Petričević",
        "Petrović",
        "Pečur",
        "Peša",
        "Pešić",
        "Pikec",
        "Piljek",
        "Pintarić",
        "Plantak",
        "Plantek",
        "Plažanin",
        "Pleše",
        "Pokas",
        "Pokos",
        "Polić",
        "Poljak",
        "Polonijo",
        "Polović",
        "Pongrac",
        "Popović",
        "Poropat",
        "Poslon",
        "Pozder",
        "Požega",
        "Predovan",
        "Prelec",
        "Preočanin",
        "Pribanić",
        "Priselac",
        "Prpić",
        "Prskalo",
        "Prtenjača",
        "Puharić",
        "Puljiz",
        "Putinja",
        "Puškarić",
        "Radelić",
        "Radin",
        "Radinović",
        "Radić",
        "Radman",
        "Radojković",
        "Radolović",
        "Radović",
        "Radočaj",
        "Radošević",
        "Raguž",
        "Rahija",
        "Rajn",
        "Rajčić",
        "Rakela",
        "Rakuljić",
        "Raljević",
        "Raspor",
        "Rastija",
        "Rađenović",
        "Rašeta",
        "Rašić",
        "Ražov",
        "Rebić",
        "Rendulić",
        "Resanović",
        "Ribarić",
        "Ribić",
        "Ribičić",
        "Rijetković",
        "Risek",
        "Ritoša",
        "Ričko",
        "Roca",
        "Roce",
        "Rogošić",
        "Rojnić",
        "Roso",
        "Rubeša",
        "Rubinić",
        "Rukavina",
        "Rumora",
        "Sabljak",
        "Sabol",
        "Sakač",
        "Salopek",
        "Sardelić",
        "Sedlar",
        "Semialjac",
        "Seničić",
        "Seršić",
        "Sever",
        "Sikirić",
        "Simić",
        "Sinožić",
        "Sirotić",
        "Skupnjak",
        "Sladonja",
        "Slavica",
        "Smoković",
        "Smolić",
        "Sobota",
        "Sokač",
        "Sokić",
        "Sokol",
        "Soldo",
        "Solomun",
        "Sorić",
        "Sošić",
        "Sršen",
        "Stanić",
        "Staničić",
        "Stančin",
        "Starčević",
        "Stipanović",
        "Stiperski",
        "Stojanov",
        "Stojnić",
        "Stojčević",
        "Stolnik",
        "Stošić",
        "Stražičić",
        "Strinavić",
        "Stupalo",
        "Surać",
        "Svetličić",
        "Tepeš",
        "Terlević",
        "Terzić",
        "Tešija",
        "Tisaj",
        "Toić",
        "Tolj",
        "Tomić",
        "Tomičić",
        "Tomljanović",
        "Tonc",
        "Topić",
        "Totić",
        "Trbović",
        "Trgovčić",
        "Triplat",
        "Trnski",
        "Trutanić",
        "Tudić",
        "Tudor",
        "Tuksar",
        "Turina",
        "Turk",
        "Turudić",
        "Turčinov",
        "Tuđa",
        "Tušek",
        "Tuškan",
        "Ugrinić",
        "Ukić",
        "Urlić",
        "Ušić",
        "Vaci",
        "Valentić",
        "Valjetić",
        "Varga",
        "Veić",
        "Vela",
        "Vidaković",
        "Vidas",
        "Vidov",
        "Vidović",
        "Viljevac",
        "Vincetić",
        "Vitasović",
        "Vižintin",
        "Vladislavić",
        "Vlašić",
        "Vojković",
        "Volarević",
        "Volarić",
        "Vorkapić",
        "Vozila",
        "Vrabelj",
        "Vranić",
        "Vrban",
        "Vretenar",
        "Vrhovec",
        "Vugdelija",
        "Vugec",
        "Vuk",
        "Vukman",
        "Vukobratović",
        "Vuković",
        "Vukušić",
        "Vuljak",
        "Vučetić",
        "Vučin",
        "Vučković",
        "Zakinja",
        "Zanoški",
        "Zeba",
        "Zebec",
        "Zelić",
        "Zgorelec",
        "Zmaić",
        "Zrilić",
        "Zrinski",
        "Zubčić",
        "Ćorić",
        "Ćosić",
        "Ćurić",
        "Čagalj",
        "Čargonja",
        "Čizmić",
        "Čiš",
        "Čižmešija",
        "Čop",
        "Čotić",
        "Čović",
        "Čubrić",
        "Čudić",
        "Čukman",
        "Čulina",
        "Čuljak",
        "Čupić",
        "Čuček",
        "Đurašević",
        "Đurinić",
        "Šalić",
        "Šantić",
        "Šargač",
        "Šarić",
        "Šarlija",
        "Šegović",
        "Šelendić",
        "Šeparović",
        "Šestak",
        "Šestan",
        "Šibalić",
        "Šimara",
        "Šimić",
        "Šimičić",
        "Šimunić",
        "Šimunović",
        "Šinković",
        "Šipek",
        "Šipić",
        "Šitum",
        "Škara",
        "Škoda",
        "Škrlin",
        "Škrnički",
        "Škrtić",
        "Škugor",
        "Škunca",
        "Šokčević",
        "Šošić",
        "Šoštarić",
        "Špika",
        "Špišić",
        "Špoljarić",
        "Špralja",
        "Štefanec",
        "Štefović",
        "Štifanić",
        "Štimac",
        "Štrbac",
        "Štrljić",
        "Šturlan",
        "Šunjić",
        "Šupraha",
        "Šuran",
        "Šurbek",
        "Šurina",
        "Šverko",
        "Žabjačan",
        "Žagar",
        "Žerjav",
        "Žeželić",
        "Žic",
        "Žiković",
        "Živić",
        "Živković",
        "Žufika",
        "Žugec",
        "Žunec",
        "Županić",
        "Žuvela",
        "Žužić",
    ]



import os
import configparser


local_path = os.path.dirname(os.path.realpath(__file__))
root_path = os.path.dirname(local_path)


class Config:
    def __init__(self):
        self.file_path = os.path.join(root_path, 'running', 'conf.ini')
        self.cf = configparser.ConfigParser()
        self.cf.read(self.file_path)

    def get_name(self, module, key):
        return self.cf.get(module, key)

    def set_name(self, module, key, value):
        self.cf.set(module, key, value)
        with open(self.file_path, 'w') as f:
            self.cf.write(f)

    def get(self, module, key):
        return self.get_name(module, key)

    def set(self, module, key, value):
        self.set_name(module, key, value)


conf = Config()


if __name__ == '__main__':
    print(conf.get_name('info', 'platform'))



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
from qrunner import __version__, __description__

try:
    long_description = open(os.path.join('qrunner', "README.md"), encoding='utf-8').read()
except IOError:
    long_description = ""

setup(
    name="qrunner",
    version=__version__,
    description=__description__,
    author="杨康",
    author_email="772840356@qq.com",
    url="https://github.com/bluepang/qrunner",
    platforms="Android,IOS,Web",
    packages=find_packages(),
    long_description=long_description,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
    ],
    include_package_data=True,
    package_data={
        r'': ['*.ini'],
    },
    install_requires=[
        'tidevice==0.6.1',
        'facebook-wda==1.4.6',
        'uiautomator2==2.16.13',
        'selenium',
        'pytest==6.2.5',
        'pytest-rerunfailures==10.2',
        'allure-pytest==2.9.45',
        'pytest-dependency==0.5.1',
        # 'pandas==1.3.4',
        # 'openpyxl==3.0.9',
        # 'XlsxWriter==3.0.2',
        # 'jira==3.1.1',
        'pytest-xdist==2.5.0',
        'webdriver-manager==3.5.2',
        'PyMySQL==0.10.1',
        'jmespath==0.9.5',
        # 'jsonschema==4.4.0',
        # 'pymongo==4.0.1',
        'pycryptodome==3.14.1',
        'python-dateutil==2.8.2',
        'pytest-ordering==0.6'
    ],
    entry_points={
        'console_scripts': [
            'qrun = qrunner.cli:main',
            'qrunner = qrunner.cli:main'
        ]
    },
)



from collections import OrderedDict

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):

    license_formats = OrderedDict(
        [
            ("???###", 0.6),
            ("???##?", 0.3),
            ("T####", 0.03),
            ("??####", 0.01),
            ("R#####", 0.03),
            ("S#####", 0.03),
        ]
    )



from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    formats = (
        # Local calls
        "%## ####",
        "%##-####",
        "%######",
        # National & Mobile dialing
        "0{{area_code}} %## ####",
        "0{{area_code}} %##-####",
        "0{{area_code}}-%##-####",
        "0{{area_code}} %######",
        # Optional parenthesis
        "(0{{area_code}}) %## ####",
        "(0{{area_code}}) %##-####",
        "(0{{area_code}}) %######",
        # International drops the 0
        "+64 {{area_code}} %## ####",
        "+64 {{area_code}} %##-####",
        "+64 {{area_code}} %######",
        "+64-{{area_code}}-%##-####",
        "+64{{area_code}}%######",
    )

    area_codes = [
        # Mobiles
        "20",
        "21",
        "22",
        "27",
        "29",
        "3",  # South Island
        "4",  # Wellington
        "6",  # Lower North Island
        "7",  # Central North Island
        "9",  # Auckland
    ]

    def area_code(self) -> str:
        return self.numerify(self.random_element(self.area_codes))

    def phone_number(self) -> str:
        pattern: str = self.random_element(self.formats)
        return self.numerify(self.generator.parse(pattern))



import os
import sys
import time
import allure
from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from selenium.webdriver import ChromeOptions
from selenium.webdriver import FirefoxOptions
from qrunner.utils.webdriver_manager_extend import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import IEDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager
from webdriver_manager.opera import OperaDriverManager
from qrunner.utils.log import logger
from qrunner.utils.config import conf
from urllib import parse


# def relaunch(func):
#     """
#     重启chromedriver的装饰器
#     :param func: 待包裹的函数
#     :return:
#     """
#     def wrapper(self, *args, **kwargs):
#         try:
#             return func(self, *args, **kwargs)
#         except requests.exceptions.ConnectionError as _:
#             logger.warning("web driver error, relaunch now.")
#             name = conf.get_name('web', 'browser_name')
#             self.d = BrowserDriver(name)
#     return wrapper


class ChromeConfig:
    headless = False
    options = None
    command_executor = ""


class FirefoxConfig:
    headless = False
    options = None
    command_executor = ""


class IEConfig:
    command_executor = ""


class EdgeConfig:
    command_executor = ""


class OperaConfig:
    command_executor = ""


class SafariConfig:
    executable_path = "/usr/bin/safaridriver"
    command_executor = ""


class Browser(object):
    """
    根据关键词初始化浏览器操作句柄，
    如'chrome、google chrome、gc'代表chrome浏览器，
    如'firefox、ff'代表火狐浏览器，
    如'internet explorer、ie、IE'代表ie浏览器，
    如'edge'代表edge浏览器，
    如'opera'代表opera浏览器，
    如'safari'代表safari浏览器
    """
    name = None

    def __new__(cls, name=None):
        cls.name = name

        if (cls.name is None) or (cls.name in ["chrome", "google chrome", "gc"]):
            return cls.chrome()
        elif cls.name in ['internet explorer', 'ie', 'IE']:
            return cls.ie()
        elif cls.name in ['firefox', 'ff']:
            return cls.firefox()
        elif cls.name == 'edge':
            return cls.edge()
        elif cls.name == 'opera':
            return cls.opera()
        elif cls.name == 'safari':
            return cls.safari()
        raise NameError(f"Not found {cls.name} browser")

    @staticmethod
    def chrome():
        if ChromeConfig.command_executor != "":
            return webdriver.Remote(command_executor=ChromeConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.CHROME.copy())

        if ChromeConfig.options is None:
            chrome_options = ChromeOptions()
        else:
            chrome_options = ChromeConfig.options
        if ChromeConfig.headless is True:
            chrome_options.add_argument('--headless')

        chrome_options.add_argument("--incognito")  # 隐身模式
        chrome_options.add_experimental_option("excludeSwitches",
                                               ['enable-automation'])  # 去除自动化控制提示
        # prefs = {'profile.managed_default_content_settings.images': 2}  # 不加载图片
        # prefs = {'profile.managed_default_content_settings.javascript': 2}  # 不加载js
        # chrome_options.add_experimental_option('prefs', prefs)
        chrome_options.page_load_strategy = 'normal'  # 设置页面加载策略

        driver = webdriver.Chrome(options=chrome_options,
                                  executable_path=ChromeDriverManager().install(),
                                  )
        if ChromeConfig.headless is True:
            driver.set_window_size(1920, 1080)
        else:
            driver.maximize_window()
        # driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        #     "source": """
        #         Object.defineProperty(navigator, 'webdriver', {
        #         get: () => undefined
        #         })"""
        # })
        return driver

    @staticmethod
    def firefox():
        if FirefoxConfig.command_executor != "":
            return webdriver.Remote(command_executor=FirefoxConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.FIREFOX.copy())

        if FirefoxConfig.options is not None:
            firefox_options = FirefoxOptions()
        else:
            firefox_options = FirefoxConfig.options
        if FirefoxConfig.headless is True:
            firefox_options.headless = True

        driver = webdriver.Firefox(options=firefox_options,
                                   executable_path=GeckoDriverManager().install())
        if FirefoxConfig.headless is True:
            driver.set_window_size(1920, 1080)
        else:
            driver.maximize_window()
        return driver

    @staticmethod
    def ie():
        if IEConfig.command_executor != "":
            return webdriver.Remote(command_executor=IEConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.INTERNETEXPLORER.copy())
        driver = webdriver.Ie(executable_path=IEDriverManager().install())
        driver.maximize_window()
        return driver

    @staticmethod
    def edge():
        if EdgeConfig.command_executor != "":
            return webdriver.Remote(command_executor=EdgeConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.EDGE.copy())
        driver = webdriver.Edge(executable_path=EdgeChromiumDriverManager(log_level=1).install())
        driver.maximize_window()
        return driver

    @staticmethod
    def opera():
        if OperaConfig.command_executor != "":
            return webdriver.Remote(command_executor=OperaConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.OPERA.copy())
        driver = webdriver.Opera(executable_path=OperaDriverManager().install())
        driver.maximize_window()
        return driver

    @staticmethod
    def safari():
        if SafariConfig.command_executor != "":
            return webdriver.Remote(command_executor=SafariConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.SAFARI.copy())
        driver = webdriver.Safari(executable_path='/usr/bin/safaridriver')
        driver.maximize_window()
        return driver


class BrowserDriver(object):
    def __init__(self, browser_name=None):
        self.d = Browser(browser_name)
        try:
            timeout = int(conf.get_name('common', 'timeout'))
        except:
            pass
        else:
            self.d.set_page_load_timeout(timeout)

    def open_url(self, url):
        if "http" not in url:
            base_url = conf.get_name('common', 'base_url')
            if 'http' in base_url:
                url = parse.urljoin(base_url, url)
            else:
                logger.debug('请设置正确的base_url')
                sys.exit()
        logger.info(f'访问: {url}')
        self.d.get(url)

    def back(self):
        logger.info('返回上一页')
        self.d.back()

    def screenshot(self, file_name):
        """
        截图并保存到预定路径并上传到allure报告
        @param file_name: foo.png or fool
        @return:
        """
        # 把文件名处理成test.png的样式
        if '.' in file_name:
            file_name = file_name.split(r'.')[0]
        # 截图并保存到当前目录的images文件夹中
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%Y%m%d%H%M%S')
        file_path = os.path.join(img_dir,
                                 f'{file_name}-{time_str}.png')
        logger.info(f'截图保存至: {file_path}')
        self.d.save_screenshot(file_path)
        # 截图并上传到allure报告
        logger.info(f'截图上传至allure报告')
        allure.attach.file(file_path,
                           attachment_type=allure.attachment_type.PNG,
                           name=f'{file_name}-{time_str}')

    @property
    def page_content(self):
        page_source = self.d.page_source
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    def max_window(self):
        logger.info('窗口设置全屏')
        self.d.maximize_window()

    def set_window(self, width, height):
        logger.info('设置窗口长宽')
        self.d.set_window_size(width, height)

    def get_windows(self):
        logger.info(f'获取当前打开的窗口列表')
        return self.d.window_handles

    def switch_window(self, old_windows):
        logger.info('切换到最新的window')
        current_windows = self.d.window_handles
        newest_window = [window for window in current_windows if window not in old_windows][0]
        self.d.switch_to.window(newest_window)

    def window_scroll(self, width, height):
        logger.info(f'设置scroll bar的宽高')
        js = "window.scrollTo({w},{h});".format(w=str(width), h=str(height))
        self.d.execute_script(js)

    def switch_to_frame(self, frame_id):
        logger.info(f'切换到frame {frame_id}')
        self.d.switch_to.frame(frame_id)

    def switch_to_frame_out(self):
        logger.info('从frame中切出来')
        self.d.switch_to.default_content()

    def execute_js(self, script, *args):
        logger.info(f'执行js脚本: \n{script}')
        self.d.execute_script(script, *args)

    def click(self, element):
        logger.info(f'点击元素: {element}')
        self.d.execute_script('arguments[0].click();', element)

    def quit(self):
        logger.info('退出浏览器')
        self.d.quit()

    def close(self):
        logger.info('关闭当前页签')
        self.d.close()

    def add_cookies(self, cookies: list):
        for cookie in cookies:
            self.d.add_cookie(cookie)

    def get_cookies(self):
        logger.info('获取cookies')
        cookies = self.d.get_cookies()
        logger.info(cookies)
        return cookies

    def get_cookie(self, name):
        logger.info(f'获取cookie: {name}')
        cookie = self.d.get_cookie(name)
        logger.info(cookie)
        return cookie

    def delete_all_cookies(self):
        logger.info('删除所有cookie')
        self.d.delete_all_cookies()

    def delete_cookie(self, name):
        logger.info(f'删除cookie: {name}')
        self.d.delete_cookie(name)

    def refresh(self):
        logger.info(f'刷新当前页')
        self.d.refresh()

    def get_title(self):
        logger.info('获取页面标题')
        title = self.d.title
        logger.info(title)
        return title

    def get_url(self):
        logger.info('获取页面url')
        url = self.d.current_url
        logger.info(url)
        return url

    def get_alert_text(self):
        logger.info('获取alert的文本')
        alert_text = self.d.switch_to.alert.text
        logger.info(alert_text)
        return alert_text

    def accept_alert(self):
        logger.info('同意确认框')
        self.d.switch_to.alert.accept()

    def dismiss_alert(self):
        logger.info('取消确认框')
        self.d.switch_to.alert.dismiss()





from .. import Provider as CompanyProvider


class Provider(CompanyProvider):

    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
    )

    company_suffixes = (
        "SRL",
        "SA",
        "SCA",
        "SNC",
        "SCS",
        "AFJ",
        "ASF",
        "CON",
        "CRL",
        "INC",
        "LOC",
        "OC1",
        "OC2",
        "OC3",
        "PFA",
        "RA",
        "SCS",
        "SPI",
        "URL",
    )

    def company_suffix(self) -> str:
        return self.random_element(self.company_suffixes)



from math import ceil

from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    A Faker provider for the Slovakian VAT IDs
    """

    vat_id_formats = ("SK##########",)

    national_id_months = ["%.2d" % i for i in range(1, 13)] + ["%.2d" % i for i in range(51, 63)]

    def vat_id(self) -> str:
        """
        http://ec.europa.eu/taxation_customs/vies/faq.html#item_11
        :return: a random Slovakian VAT ID
        """

        return self.bothify(self.random_element(self.vat_id_formats))

    def birth_number(self) -> str:
        """
        Birth Number (Czech/Slovak: rodné číslo (RČ))
        https://en.wikipedia.org/wiki/National_identification_number#Czech_Republic_and_Slovakia
        """
        birthdate = self.generator.date_of_birth()
        year = f"{birthdate:%y}"
        month: str = self.random_element(self.national_id_months)
        day = f"{birthdate:%d}"
        if birthdate.year > 1953:
            sn = self.random_number(4, True)
        else:
            sn = self.random_number(3, True)
        number = int(f"{year}{month}{day}{sn}")
        birth_number = str(ceil(number / 11) * 11)
        if year == "00":
            birth_number = "00" + birth_number
        elif year[0] == "0":
            birth_number = "0" + birth_number
        return f"{birth_number[:6]}/{birth_number[6::]}"



from ipaddress import IPV4LENGTH, IPV6LENGTH, IPv4Network, IPv6Address, IPv6Network
from typing import Dict, List, Optional, Tuple

from ...decode import unidecode
from ...utils.decorators import lowercase, slugify, slugify_unicode
from ...utils.distribution import choices_distribution
from .. import BaseProvider, ElementsType

localized = True


class _IPv4Constants:
    """
    IPv4 network constants used to group networks into different categories.
    Structure derived from `ipaddress._IPv4Constants`.

    Excluded network list is updated to comply with current IANA list of
    private and reserved networks.
    """

    _network_classes: Dict[str, IPv4Network] = {
        "a": IPv4Network("0.0.0.0/1"),
        "b": IPv4Network("128.0.0.0/2"),
        "c": IPv4Network("192.0.0.0/3"),
    }

    # Three common private networks from class A, B and CIDR
    # to generate private addresses from.
    _private_networks: List[IPv4Network] = [
        IPv4Network("10.0.0.0/8"),
        IPv4Network("172.16.0.0/12"),
        IPv4Network("192.168.0.0/16"),
    ]

    # List of networks from which IP addresses will never be generated,
    # includes other private IANA and reserved networks from
    # https://www.iana.org/assignments/iana-ipv4-special-registry/iana-ipv4-special-registry.xhtml
    _excluded_networks: List[IPv4Network] = [
        IPv4Network("0.0.0.0/8"),
        IPv4Network("100.64.0.0/10"),
        IPv4Network("127.0.0.0/8"),  # loopback network
        IPv4Network("169.254.0.0/16"),  # linklocal network
        IPv4Network("192.0.0.0/24"),
        IPv4Network("192.0.2.0/24"),
        IPv4Network("192.31.196.0/24"),
        IPv4Network("192.52.193.0/24"),
        IPv4Network("192.88.99.0/24"),
        IPv4Network("192.175.48.0/24"),
        IPv4Network("198.18.0.0/15"),
        IPv4Network("198.51.100.0/24"),
        IPv4Network("203.0.113.0/24"),
        IPv4Network("224.0.0.0/4"),  # multicast network
        IPv4Network("240.0.0.0/4"),
        IPv4Network("255.255.255.255/32"),
    ]


class Provider(BaseProvider):
    safe_domain_names: ElementsType = ("example.org", "example.com", "example.net")
    free_email_domains: ElementsType = ("gmail.com", "yahoo.com", "hotmail.com")
    tlds: ElementsType = (
        "com",
        "com",
        "com",
        "com",
        "com",
        "com",
        "biz",
        "info",
        "net",
        "org",
    )
    hostname_prefixes: ElementsType = (
        "db",
        "srv",
        "desktop",
        "laptop",
        "lt",
        "email",
        "web",
    )
    uri_pages: ElementsType = (
        "index",
        "home",
        "search",
        "main",
        "post",
        "homepage",
        "category",
        "register",
        "login",
        "faq",
        "about",
        "terms",
        "privacy",
        "author",
    )
    uri_paths: ElementsType = (
        "app",
        "main",
        "wp-content",
        "search",
        "category",
        "tag",
        "categories",
        "tags",
        "blog",
        "posts",
        "list",
        "explore",
    )
    uri_extensions: ElementsType = (
        ".html",
        ".html",
        ".html",
        ".htm",
        ".htm",
        ".php",
        ".php",
        ".jsp",
        ".asp",
    )
    http_methods: ElementsType = (
        "GET",
        "HEAD",
        "POST",
        "PUT",
        "DELETE",
        "CONNECT",
        "OPTIONS",
        "TRACE",
        "PATCH",
    )

    user_name_formats: ElementsType = (
        "{{last_name}}.{{first_name}}",
        "{{first_name}}.{{last_name}}",
        "{{first_name}}##",
        "?{{last_name}}",
    )
    email_formats: ElementsType = (
        "{{user_name}}@{{domain_name}}",
        "{{user_name}}@{{free_email_domain}}",
    )
    url_formats: ElementsType = (
        "www.{{domain_name}}/",
        "{{domain_name}}/",
    )
    uri_formats: ElementsType = (
        "{{url}}",
        "{{url}}{{uri_page}}/",
        "{{url}}{{uri_page}}{{uri_extension}}",
        "{{url}}{{uri_path}}/{{uri_page}}/",
        "{{url}}{{uri_path}}/{{uri_page}}{{uri_extension}}",
    )
    image_placeholder_services: ElementsType = (
        "https://picsum.photos/{width}/{height}",
        "https://dummyimage.com/{width}x{height}",
        "https://placekitten.com/{width}/{height}",
        "https://placeimg.com/{width}/{height}/any",
    )

    replacements: Tuple[Tuple[str, str], ...] = ()

    def _to_ascii(self, string: str) -> str:
        for search, replace in self.replacements:
            string = string.replace(search, replace)

        string = unidecode(string)
        return string

    @lowercase
    def email(self, safe: bool = True, domain: Optional[str] = None) -> str:
        if domain:
            email = f"{self.user_name()}@{domain}"
        elif safe:
            email = f"{self.user_name()}@{self.safe_domain_name()}"
        else:
            pattern: str = self.random_element(self.email_formats)
            email = "".join(self.generator.parse(pattern).split(" "))
        return email

    @lowercase
    def safe_domain_name(self) -> str:
        return self.random_element(self.safe_domain_names)

    @lowercase
    def safe_email(self) -> str:
        return self.user_name() + "@" + self.safe_domain_name()

    @lowercase
    def free_email(self) -> str:
        return self.user_name() + "@" + self.free_email_domain()

    @lowercase
    def company_email(self) -> str:
        return self.user_name() + "@" + self.domain_name()

    @lowercase
    def free_email_domain(self) -> str:
        return self.random_element(self.free_email_domains)

    @lowercase
    def ascii_email(self) -> str:
        pattern: str = self.random_element(self.email_formats)
        return self._to_ascii(
            "".join(self.generator.parse(pattern).split(" ")),
        )

    @lowercase
    def ascii_safe_email(self) -> str:
        return self._to_ascii(self.user_name() + "@" + self.safe_domain_name())

    @lowercase
    def ascii_free_email(self) -> str:
        return self._to_ascii(
            self.user_name() + "@" + self.free_email_domain(),
        )

    @lowercase
    def ascii_company_email(self) -> str:
        return self._to_ascii(
            self.user_name() + "@" + self.domain_name(),
        )

    @slugify_unicode
    def user_name(self) -> str:
        pattern: str = self.random_element(self.user_name_formats)
        return self._to_ascii(self.bothify(self.generator.parse(pattern)).lower())

    @lowercase
    def hostname(self, levels: int = 1) -> str:
        """
        Produce a hostname with specified number of subdomain levels.

        >>> hostname()
        db-01.nichols-phillips.com
        >>> hostname(0)
        laptop-56
        >>> hostname(2)
        web-12.williamson-hopkins.jackson.com
        """
        hostname_prefix: str = self.random_element(self.hostname_prefixes)
        hostname_prefix_first_level: str = hostname_prefix + "-" + self.numerify("##")
        return (
            hostname_prefix_first_level if levels < 1 else hostname_prefix_first_level + "." + self.domain_name(levels)
        )

    @lowercase
    def domain_name(self, levels: int = 1) -> str:
        """
        Produce an Internet domain name with the specified number of
        subdomain levels.

        >>> domain_name()
        nichols-phillips.com
        >>> domain_name(2)
        williamson-hopkins.jackson.com
        """
        if levels < 1:
            raise ValueError("levels must be greater than or equal to 1")
        if levels == 1:
            return self.domain_word() + "." + self.tld()
        return self.domain_word() + "." + self.domain_name(levels - 1)

    @lowercase
    @slugify_unicode
    def domain_word(self) -> str:
        company: str = self.generator.format("company")
        company_elements: List[str] = company.split(" ")
        return self._to_ascii(company_elements.pop(0))

    def dga(
        self,
        year: Optional[int] = None,
        month: Optional[int] = None,
        day: Optional[int] = None,
        tld: Optional[str] = None,
        length: Optional[int] = None,
    ) -> str:
        """Generates a domain name by given date
        https://en.wikipedia.org/wiki/Domain_generation_algorithm

        :type year: int
        :type month: int
        :type day: int
        :type tld: str
        :type length: int
        :rtype: str
        """

        domain = ""
        year = year or self.random_int(min=1, max=9999)
        month = month or self.random_int(min=1, max=12)
        day = day or self.random_int(min=1, max=30)
        tld = tld or self.tld()
        length = length or self.random_int(min=2, max=63)

        for _ in range(length):
            year = ((year ^ 8 * year) >> 11) ^ ((year & 0xFFFFFFF0) << 17)
            month = ((month ^ 4 * month) >> 25) ^ 16 * (month & 0xFFFFFFF8)
            day = ((day ^ (day << 13)) >> 19) ^ ((day & 0xFFFFFFFE) << 12)
            domain += chr(((year ^ month ^ day) % 25) + 97)

        return domain + "." + tld

    def tld(self) -> str:
        return self.random_element(self.tlds)

    def http_method(self) -> str:
        """Returns random HTTP method
        https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods

        :rtype: str
        """

        return self.random_element(self.http_methods)

    def url(self, schemes: Optional[List[str]] = None) -> str:
        """
        :param schemes: a list of strings to use as schemes, one will chosen randomly.
            If None, it will generate http and https urls.
            Passing an empty list will result in schemeless url generation like "://domain.com".
        :return: a random url string.

        """
        if schemes is None:
            schemes = ["http", "https"]

        pattern: str = f'{self.random_element(schemes) if schemes else ""}://{self.random_element(self.url_formats)}'

        return self.generator.parse(pattern)

    def _get_all_networks_and_weights(self, address_class: Optional[str] = None) -> Tuple[List[IPv4Network], List[int]]:
        """
        Produces a 2-tuple of valid IPv4 networks and corresponding relative weights

        :param address_class: IPv4 address class (a, b, or c)
        """
        # If `address_class` has an unexpected value, use the whole IPv4 pool
        if address_class in _IPv4Constants._network_classes.keys():
            networks_attr = f"_cached_all_class_{address_class}_networks"
            all_networks = [_IPv4Constants._network_classes[address_class]]  # type: ignore
        else:
            networks_attr = "_cached_all_networks"
            all_networks = [IPv4Network("0.0.0.0/0")]

        # Return cached network and weight data if available
        weights_attr = f"{networks_attr}_weights"
        if hasattr(self, networks_attr) and hasattr(self, weights_attr):
            return getattr(self, networks_attr), getattr(self, weights_attr)

        # Otherwise, compute for list of networks (excluding special networks)
        all_networks = self._exclude_ipv4_networks(
            all_networks,
            _IPv4Constants._excluded_networks,
        )

        # Then compute for list of corresponding relative weights
        weights = [network.num_addresses for network in all_networks]

        # Then cache and return results
        setattr(self, networks_attr, all_networks)
        setattr(self, weights_attr, weights)
        return all_networks, weights

    def _get_private_networks_and_weights(
        self,
        address_class: Optional[str] = None,
    ) -> Tuple[List[IPv4Network], List[int]]:
        """
        Produces an OrderedDict of valid private IPv4 networks and corresponding relative weights

        :param address_class: IPv4 address class (a, b, or c)
        """
        # If `address_class` has an unexpected value, choose a valid value at random
        if not address_class or address_class not in _IPv4Constants._network_classes.keys():
            address_class = self.ipv4_network_class()

        # Return cached network and weight data if available for a specific address class
        networks_attr = f"_cached_private_class_{address_class}_networks"
        weights_attr = f"{networks_attr}_weights"
        if hasattr(self, networks_attr) and hasattr(self, weights_attr):
            return getattr(self, networks_attr), getattr(self, weights_attr)

        # Otherwise, compute for list of private networks (excluding special networks)
        supernet = _IPv4Constants._network_classes[address_class]
        private_networks = [subnet for subnet in _IPv4Constants._private_networks if subnet.overlaps(supernet)]
        private_networks = self._exclude_ipv4_networks(
            private_networks,
            _IPv4Constants._excluded_networks,
        )

        # Then compute for list of corresponding relative weights
        weights = [network.num_addresses for network in private_networks]

        # Then cache and return results
        setattr(self, networks_attr, private_networks)
        setattr(self, weights_attr, weights)
        return private_networks, weights

    def _get_public_networks_and_weights(
        self,
        address_class: Optional[str] = None,
    ) -> Tuple[List[IPv4Network], List[int]]:
        """
        Produces a 2-tuple of valid public IPv4 networks and corresponding relative weights

        :param address_class: IPv4 address class (a, b, or c)
        """
        # If `address_class` has an unexpected value, choose a valid value at random
        if address_class not in _IPv4Constants._network_classes.keys():
            address_class = self.ipv4_network_class()

        # Return cached network and weight data if available for a specific address class
        networks_attr = f"_cached_public_class_{address_class}_networks"
        weights_attr = f"{networks_attr}_weights"
        if hasattr(self, networks_attr) and hasattr(self, weights_attr):
            return getattr(self, networks_attr), getattr(self, weights_attr)

        # Otherwise, compute for list of public networks (excluding private and special networks)
        public_networks = [_IPv4Constants._network_classes[address_class]]  # type: ignore
        public_networks = self._exclude_ipv4_networks(
            public_networks,
            _IPv4Constants._private_networks + _IPv4Constants._excluded_networks,
        )

        # Then compute for list of corresponding relative weights
        weights = [network.num_addresses for network in public_networks]

        # Then cache and return results
        setattr(self, networks_attr, public_networks)
        setattr(self, weights_attr, weights)
        return public_networks, weights

    def _random_ipv4_address_from_subnets(
        self,
        subnets: List[IPv4Network],
        weights: Optional[List[int]] = None,
        network: bool = False,
    ) -> str:
        """
        Produces a random IPv4 address or network with a valid CIDR
        from within the given subnets using a distribution described
        by weights.

        :param subnets: List of IPv4Networks to choose from within
        :param weights: List of weights corresponding to the individual IPv4Networks
        :param network: Return a network address, and not an IP address
        :return:
        """
        if not subnets:
            raise ValueError("No subnets to choose from")

        # If the weights argument has an invalid value, default to equal distribution
        if (
            isinstance(weights, list)
            and len(subnets) == len(weights)
            and all(isinstance(w, (float, int)) for w in weights)
        ):
            subnet = choices_distribution(
                subnets,
                [float(w) for w in weights],
                random=self.generator.random,
                length=1,
            )[0]
        else:
            subnet = self.generator.random.choice(subnets)

        address = str(
            subnet[
                self.generator.random.randint(
                    0,
                    subnet.num_addresses - 1,
                )
            ],
        )

        if network:
            address += "/" + str(
                self.generator.random.randint(
                    subnet.prefixlen,
                    subnet.max_prefixlen,
                )
            )
            address = str(IPv4Network(address, strict=False))

        return address

    def _exclude_ipv4_networks(
        self, networks: List[IPv4Network], networks_to_exclude: List[IPv4Network]
    ) -> List[IPv4Network]:
        """
        Exclude the list of networks from another list of networks
        and return a flat list of new networks.

        :param networks: List of IPv4 networks to exclude from
        :param networks_to_exclude: List of IPv4 networks to exclude
        :returns: Flat list of IPv4 networks
        """
        networks_to_exclude.sort(key=lambda x: x.prefixlen)
        for network_to_exclude in networks_to_exclude:

            def _exclude_ipv4_network(network):
                """
                Exclude a single network from another single network
                and return a list of networks. Network to exclude
                comes from the outer scope.

                :param network: Network to exclude from
                :returns: Flat list of IPv4 networks after exclusion.
                          If exclude fails because networks do not
                          overlap, a single element list with the
                          orignal network is returned. If it overlaps,
                          even partially, the network is excluded.
                """
                try:
                    return list(network.address_exclude(network_to_exclude))
                except ValueError:
                    # If networks overlap partially, `address_exclude`
                    # will fail, but the network still must not be used
                    # in generation.
                    if network.overlaps(network_to_exclude):
                        return []
                    else:
                        return [network]

            nested_networks = list(map(_exclude_ipv4_network, networks))
            networks = [item for nested in nested_networks for item in nested]

        return networks

    def ipv4_network_class(self) -> str:
        """
        Returns a IPv4 network class 'a', 'b' or 'c'.

        :returns: IPv4 network class
        """
        return self.random_element("abc")

    def ipv4(
        self,
        network: bool = False,
        address_class: Optional[str] = None,
        private: Optional[str] = None,
    ) -> str:
        """
        Returns a random IPv4 address or network with a valid CIDR.

        :param network: Network address
        :param address_class: IPv4 address class (a, b, or c)
        :param private: Public or private
        :returns: IPv4
        """
        if private is True:
            return self.ipv4_private(address_class=address_class, network=network)
        elif private is False:
            return self.ipv4_public(address_class=address_class, network=network)
        else:
            all_networks, weights = self._get_all_networks_and_weights(address_class=address_class)
            return self._random_ipv4_address_from_subnets(all_networks, weights=weights, network=network)

    def ipv4_private(self, network: bool = False, address_class: Optional[str] = None) -> str:
        """
        Returns a private IPv4.

        :param network: Network address
        :param address_class: IPv4 address class (a, b, or c)
        :returns: Private IPv4
        """
        private_networks, weights = self._get_private_networks_and_weights(address_class=address_class)
        return self._random_ipv4_address_from_subnets(private_networks, weights=weights, network=network)

    def ipv4_public(self, network: bool = False, address_class: Optional[str] = None) -> str:
        """
        Returns a public IPv4 excluding private blocks.

        :param network: Network address
        :param address_class: IPv4 address class (a, b, or c)
        :returns: Public IPv4
        """
        public_networks, weights = self._get_public_networks_and_weights(address_class=address_class)
        return self._random_ipv4_address_from_subnets(public_networks, weights=weights, network=network)

    def ipv6(self, network: bool = False) -> str:
        """Produce a random IPv6 address or network with a valid CIDR"""
        address = str(IPv6Address(self.generator.random.randint(2**IPV4LENGTH, (2**IPV6LENGTH) - 1)))
        if network:
            address += "/" + str(self.generator.random.randint(0, IPV6LENGTH))
            address = str(IPv6Network(address, strict=False))
        return address

    def mac_address(self) -> str:
        mac = [self.generator.random.randint(0x00, 0xFF) for _ in range(0, 6)]
        return ":".join(map(lambda x: "%02x" % x, mac))

    def port_number(self, is_system: bool = False, is_user: bool = False, is_dynamic: bool = False) -> int:
        """Returns a network port number
        https://tools.ietf.org/html/rfc6335

        :param is_system: System or well-known ports
        :param is_user: User or registered ports
        :param is_dynamic: Dynamic / private / ephemeral ports
        :rtype: int
        """

        if is_system:
            return self.random_int(min=0, max=1023)
        elif is_user:
            return self.random_int(min=1024, max=49151)
        elif is_dynamic:
            return self.random_int(min=49152, max=65535)

        return self.random_int(min=0, max=65535)

    def uri_page(self) -> str:
        return self.random_element(self.uri_pages)

    def uri_path(self, deep: Optional[int] = None) -> str:
        deep = deep if deep else self.generator.random.randint(1, 3)
        return "/".join(
            self.random_elements(self.uri_paths, length=deep),
        )

    def uri_extension(self) -> str:
        return self.random_element(self.uri_extensions)

    def uri(self) -> str:
        pattern: str = self.random_element(self.uri_formats)
        return self.generator.parse(pattern)

    @slugify
    def slug(self, value: Optional[str] = None) -> str:
        """Django algorithm"""
        if value is None:
            value = self.generator.text(20)
        return value

    def image_url(
        self,
        width: Optional[int] = None,
        height: Optional[int] = None,
        placeholder_url: Optional[str] = None,
    ) -> str:
        """
        Returns URL to placeholder image
        Example: http://placehold.it/640x480

        :param width: Optional image width
        :param height: Optional image height
        :param placeholder_url: Optional template string of image URLs from custom
            placeholder service. String must contain ``{width}`` and ``{height}``
            placeholders, eg: ``https:/example.com/{width}/{height}``.
        :rtype: str
        """
        width_ = width or self.random_int(max=1024)
        height_ = height or self.random_int(max=1024)
        if placeholder_url is None:
            placeholder_url = self.random_element(self.image_placeholder_services)
        return placeholder_url.format(width=width_, height=height_)

    def iana_id(self) -> str:
        """Returns IANA Registrar ID
        https://www.iana.org/assignments/registrar-ids/registrar-ids.xhtml

        :rtype: str
        """

        return str(self.random_int(min=1, max=8888888))

    def ripe_id(self) -> str:
        """Returns RIPE Organization ID
        https://www.ripe.net/manage-ips-and-asns/db/support/organisation-object-in-the-ripe-database

        :rtype: str
        """

        lex = "?" * self.random_int(min=2, max=4)
        num = "%" * self.random_int(min=1, max=5)
        return self.bothify(f"ORG-{lex}{num}-RIPE").upper()

    def nic_handle(self, suffix: str = "FAKE") -> str:
        """Returns NIC Handle ID
        https://www.apnic.net/manage-ip/using-whois/guide/person/

        :rtype: str
        """

        if len(suffix) < 2:
            raise ValueError("suffix length must be greater than or equal to 2")

        lex = "?" * self.random_int(min=2, max=4)
        num = "%" * self.random_int(min=1, max=5)
        return self.bothify(f"{lex}{num}-{suffix}").upper()

    def nic_handles(self, count: int = 1, suffix: str = "????") -> List[str]:
        """Returns NIC Handle ID list

        :rtype: list[str]
        """

        return [self.nic_handle(suffix=suffix) for _ in range(count)]



import inspect
from typing import Union
from wda import Selector
from qrunner.utils.log import logger
from qrunner.utils.config import conf
from qrunner.core.ios.driver import IosDriver
from multiprocessing.dummy import Pool as ThreadPool
from selenium.common.exceptions import NoSuchElementException


class IosElement(object):
    """
    IOS原生元素定义
    """
    def __init__(self, driver=None, *args, **kwargs):
        # 从kwargs中删掉并返回index
        self._index = kwargs.pop('index', 0)
        # 参数初始化
        self._xpath = kwargs.get('xpath', '')
        self._kwargs = kwargs
        self._element: Union[Selector]
        self._driver: IosDriver = driver
        self._serial_no = self._driver.serial_no
        self._pkg_name = self._driver.bundle_id
        self._d = self._driver.d

    # 多线程处理异常弹窗
    def handle_alert(self):
        """
        根据不同定位方式进行点击
        @return:
        """
        def click_alert(loc):
            if '//' in loc:
                element = self._d.xpath(loc)
            else:
                element = self._d(label=loc)
            element.click_exists(timeout=1)

        # 多线程执行点击过程
        alert_str = conf.get_name('app', 'ios_alert')
        logger.debug(f'处理异常弹窗: {alert_str}')
        try:
            alert_list = alert_str.split(',')
        except:
            alert_list = []
        if alert_list:
            pool = ThreadPool(10)
            pool.map(click_alert, alert_list)

    def _find_element(self, retry=3, timeout=3):
        """
        循环查找元素，查找失败先处理弹窗后重试
        @param retry: 重试次数
        @param timeout: 单次查找超时时间
        @return:
        """
        self._element = self._d.xpath(self._xpath) if \
            self._xpath else self._d(**self._kwargs)[self._index]
        self.handle_alert()
        while not self._element.wait(timeout=timeout):
            if retry > 0:
                retry -= 1
                logger.warning(f'重试 查找元素： {self._kwargs}')
                self.handle_alert()
            else:
                frame = inspect.currentframe().f_back
                caller = inspect.getframeinfo(frame)
                logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
                return None
        return self._element

    def get_element(self, retry=3, timeout=3):
        """
        针对元素定位失败的情况，抛出NoSuchElementException异常
        @param retry:
        @param timeout:
        @return:
        """
        element: Union[Selector] = self._find_element(retry=retry, timeout=timeout)
        if element is None:
            self._driver.screenshot(f'元素: {self._kwargs}-定位失败')
            raise NoSuchElementException(f'元素: {self._kwargs} 定位失败')
        else:
            self._driver.screenshot(f'元素: {self._kwargs}-定位成功')
        return element

    @property
    def info(self):
        logger.info(f'获取元素: {self._kwargs} 的所有属性')
        return self.get_element().info

    @property
    def text(self):
        logger.info(f'获取元素: {self._kwargs} 的text属性')
        return self.get_element().text

    @property
    def className(self):
        logger.info(f'获取元素: {self._kwargs} 的className属性')
        return self.get_element().className

    @property
    def name(self):
        logger.info(f'获取元素: {self._kwargs} 的name属性')
        return self.get_element().name

    @property
    def visible(self):
        logger.info(f'获取元素: {self._kwargs} 的visible属性')
        return self.get_element().visible

    @property
    def value(self):
        logger.info(f'获取元素: {self._kwargs} 的value属性')
        return self.get_element().value

    @property
    def label(self):
        logger.info(f'获取元素: {self._kwargs} 的label属性')
        return self.get_element().label

    @property
    def enabled(self):
        logger.info(f'获取元素: {self._kwargs} 的enabled属性')
        return self.get_element().enabled

    @property
    def displayed(self):
        logger.info(f'获取元素: {self._kwargs} 的displayed属性')
        return self.get_element().displayed

    @property
    def bounds(self):
        logger.info(f'获取元素: {self._kwargs} 的bounds属性')
        return self.get_element().bounds

    def exists(self, timeout=1):
        """
        判断元素是否存在当前页面
        @param timeout:
        @return:
        """
        logger.info(f'元素 {self._kwargs},{self._index} 是否存在:')
        element = self._find_element(retry=0, timeout=timeout)
        if element is None:
            self._driver.screenshot(f'元素定位失败')
            return False
        return True

    def wait_gone(self, timeout=10):
        logger.info(f'等待元素{self._kwargs}消失')
        flag = self.get_element().wait_gone(timeout=timeout)
        logger.info(flag)
        return flag

    def click(self):
        logger.info(f'点击元素: {self._kwargs},{self._index}')
        self.get_element().click()

    def click_exists(self):
        if self.exists():
            self.click()

    def clear_text(self):
        logger.info('清除文本: {text}')
        self.get_element().clear_text()

    def set_text(self, text):
        logger.info(f'输入框 {self._kwargs},{self._index} 输入: {text}')
        self.get_element().set_text(text)

    def scroll(self, direction=None):
        """
        scroll to make element visiable
        @param: direction，方向，"up", "down", "left", "right"
        @return:
        """
        if direction is not None:
            self.get_element().scroll(direction=direction)
        else:
            self.get_element().scroll()

    def swipe_left(self):
        self.get_element().swipe("left")

    def swipe_right(self):
        self.get_element().swipe("right")

    def swipe_up(self):
        self.get_element().swipe("up")

    def swipe_down(self):
        self.get_element().swipe("down")

    def child(self, *args, **kwargs):
        return self.get_element().child(*args, **kwargs)









from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
    )

    company_suffixes = (
        "s.r.o.",
        "o.s.",
        "a.s.",
    )
